import { Action } from '@ngrx/store';

import { RESTClientError } from '@loafer/ng-rest';

import { Member } from '@overflow/commons-typescript/model/member';
import { DomainMember } from '@overflow/commons-typescript/model/domain';

export enum ActionType {
  Signin = '[member.member] Signin',
  SigninSuccess = '[member.member] SigninSuccess',
  SigninFailure = '[member.member] SigninFailure',
  SigninRedirect = '[member.member] SigninRedirect',

  SigninCookie = '[member.auth] SigninCookie',
  SigninCookieSuccess = '[member.auth] SigninCookieSuccess',
  SigninCookieFailure = '[member.auth] SigninCookieFailure',

  Signout = '[member.auth] Signout',
  SignoutSuccess = '[member.auth] SignoutSuccess',
  SignoutFailure = '[member.auth] SignoutFailure',

  Signup = '[member.signup] Signup',
  SignupSuccess = '[member.signup] SignupSuccess',
  SignupFailure = '[member.signup] SignupFailure',

  Modify = '[member.modify] Modify',
  ModifySuccess = '[member.modify] ModifySuccess',
  ModifyFailure = '[member.modify] ModifyFailure',

  ResetPassword = '[member.resetPassword] ResetPassword',
  ResetPasswordSuccess = '[member.resetPassword] ResetPasswordSuccess',
  ResetPasswordFailure = '[member.resetPassword] ResetPasswordFailure',

  ModifyPassword = '[member.modifyPassword] ModifyPassword',
  ModifyPasswordSuccess = '[member.modifyPassword] ModifyPasswordSuccess',
  ModifyPasswordFailure = '[member.modifyPassword] ModifyPasswordFailure',
}

export class Signin implements Action {
  readonly type = ActionType.Signin;

  constructor(public payload: { email: string, password: string, returnURL: string }) { }
}

export class SigninSuccess implements Action {
  readonly type = ActionType.SigninSuccess;

  constructor(public payload: { authToken: string, domainMember: DomainMember }) { }
}

export class SigninFailure implements Action {
  readonly type = ActionType.SigninFailure;

  constructor(public payload: RESTClientError) { }
}

export class SigninRedirect implements Action {
  readonly type = ActionType.SigninRedirect;
  constructor(public payload: string) { }
}

export class SigninCookie implements Action {
  readonly type = ActionType.SigninCookie;

  constructor(public payload: { authToken: string, returnURL: string }) { }
}

export class SigninCookieSuccess implements Action {
  readonly type = ActionType.SigninCookieSuccess;

  constructor(public payload: DomainMember) { }
}

export class SigninCookieFailure implements Action {
  readonly type = ActionType.SigninCookieFailure;

  constructor(public payload: RESTClientError) { }
}

export class Signout implements Action {
  readonly type = ActionType.Signout;
}

export class SignoutSuccess implements Action {
  readonly type = ActionType.SignoutSuccess;
}

export class SignoutFailure implements Action {
  readonly type = ActionType.SignoutFailure;

  constructor(public payload: RESTClientError) { }
}

export class Signup implements Action {
  readonly type = ActionType.Signup;

  constructor(public payload: { member: Member, password: string }) { }
}

export class SignupSuccess implements Action {
  readonly type = ActionType.SignupSuccess;

  constructor(public payload: Member) { }
}

export class SignupFailure implements Action {
  readonly type = ActionType.SignupFailure;

  constructor(public payload: RESTClientError) { }
}

export class Modify implements Action {
  readonly type = ActionType.Modify;

  constructor(public payload: Member ) {}
}

export class ModifySuccess implements Action {
  readonly type = ActionType.ModifySuccess;

  constructor(public payload: Member) {}
}

export class ModifyFailure implements Action {
  readonly type = ActionType.ModifyFailure;

  constructor(public payload: RESTClientError) {}
}

export class ResetPassword implements Action {
  readonly type = ActionType.ResetPassword;

  constructor(public payload: string ) {}
}

export class ResetPasswordSuccess implements Action {
  readonly type = ActionType.ResetPasswordSuccess;

  constructor(public payload: Member) {}
}

export class ResetPasswordFailure implements Action {
  readonly type = ActionType.ResetPasswordFailure;

  constructor(public payload: RESTClientError) {}
}

export class ModifyPassword implements Action {
  readonly type = ActionType.ModifyPassword;

  constructor(public payload: { token: string, pw: string, confirmPw: string } ) {}
}

export class ModifyPasswordSuccess implements Action {
  readonly type = ActionType.ModifyPasswordSuccess;

  constructor(public payload: Member) {}
}

export class ModifyPasswordFailure implements Action {
  readonly type = ActionType.ModifyPasswordFailure;

  constructor(public payload: RESTClientError) {}
}


export type Actions =
  | Signin
  | SigninSuccess
  | SigninFailure
  | SigninRedirect
  | SigninCookie
  | SigninCookieSuccess
  | SigninCookieFailure
  | Signout
  | SignoutSuccess
  | SignoutFailure
  | Signup
  | SignupSuccess
  | SignupFailure
  | Modify
  | ModifySuccess
  | ModifyFailure
  | ResetPassword
  | ResetPasswordSuccess
  | ResetPasswordFailure
  | ModifyPassword
  | ModifyPasswordSuccess
  | ModifyPasswordFailure
  ;
